import { FilterPattern } from '@rollup/pluginutils';
import { TransformResult } from 'unplugin';
import { Awaitable } from '@antfu/utils';

interface ImportInfo {
    name?: string;
    importName?: string;
    path: string;
}
declare type SideEffectsInfo = (ImportInfo | string)[] | ImportInfo | string | undefined;
interface ComponentInfo extends ImportInfo {
    sideEffects?: SideEffectsInfo;
}
declare type ComponentResolveResult = Awaitable<string | ComponentInfo | null | undefined | void>;
declare type ComponentResolverFunction = (name: string) => ComponentResolveResult;
declare type ComponentResolverObject = {
    type: 'component' | 'directive';
    resolve: ComponentResolverFunction;
};
declare type ComponentResolver = ComponentResolverFunction | ComponentResolverObject;
interface UILibraryOptions {
    name: string;
    prefix?: string;
    entries?: string[];
}
declare type Matcher = (id: string) => boolean | null | undefined;
declare type Transformer = (code: string, id: string, path: string, query: Record<string, string>) => Awaitable<TransformResult | null>;
declare type SupportedTransformer = 'vue3' | 'vue2';
/**
 * Plugin options.
 */
interface Options {
    /**
     * RegExp or glob to match files to be transformed
     */
    include?: FilterPattern;
    /**
     * RegExp or glob to match files to NOT be transformed
     */
    exclude?: FilterPattern;
    /**
     * Relative paths to the directory to search for components.
     * @default 'src/components'
     */
    dirs?: string | string[];
    /**
     * Valid file extensions for components.
     * @default ['vue']
     */
    extensions?: string | string[];
    /**
     * Glob patterns to match file names to be detected as components.
     *
     * When specified, the `dirs` and `extensions` options will be ignored.
     */
    globs?: string | string[];
    /**
     * Search for subdirectories
     * @default true
     */
    deep?: boolean;
    /**
     * Allow subdirectories as namespace prefix for components
     * @default false
     */
    directoryAsNamespace?: boolean;
    /**
     * Subdirectory paths for ignoring namespace prefixes
     * works when `directoryAsNamespace: true`
     * @default "[]"
     */
    globalNamespaces?: string[];
    /**
     * comp libraries to use auto import
     */
    libraries?: (string | UILibraryOptions)[];
    /**
     * Pass a custom function to resolve the component importing path from the component name.
     *
     * The component names are always in PascalCase
     */
    resolvers?: (ComponentResolver | ComponentResolver[])[];
    /**
     * Apply custom transform over the path for importing
     */
    importPathTransform?: (path: string) => string | undefined;
    /**
     * Transformer to apply
     *
     * @default 'vue3'
     */
    transformer?: SupportedTransformer;
    /**
     * Generate TypeScript declaration for global components
     *
     * Accept boolean or a path related to project root
     *
     * @see https://github.com/vuejs/vue-next/pull/3399
     * @see https://github.com/johnsoncodehk/volar#using
     * @default true
     */
    dts?: boolean | string;
    /**
     * Do not emit warning on component overriding
     *
     * @default false
     */
    allowOverrides?: boolean;
    /**
     * auto import for directives.
     *
     * default: `true` for Vue 3, `false` for Vue 2
     *
     * Babel is needed to do the transformation for Vue 2, it's disabled by default for performance concerns.
     * To install Babel, run: `npm install -D @babel/parser @babel/traverse`
     * @default undefined
     */
    directives?: boolean;
}
declare type ResolvedOptions = Omit<Required<Options>, 'resolvers' | 'libraries' | 'extensions' | 'dirs' | 'globalComponentsDeclaration'> & {
    resolvers: ComponentResolverObject[];
    libraries: UILibraryOptions[];
    extensions: string[];
    dirs: string[];
    resolvedDirs: string[];
    globs: string[];
    dts: string | false;
    root: string;
};
declare type ComponentsImportMap = Record<string, string[] | undefined>;

export { ComponentInfo, ComponentResolveResult, ComponentResolver, ComponentResolverFunction, ComponentResolverObject, ComponentsImportMap, ImportInfo, Matcher, Options, ResolvedOptions, SideEffectsInfo, SupportedTransformer, Transformer, UILibraryOptions };
